package net.ihe.gazelle.proxy.netty.protocols.tlsRaw;

import net.ihe.gazelle.security.ssl.GazelleMessageParser;

import java.util.*;

/**
 * Created by gthomazon on 19/09/16.
 */
public class HandleReceivedMessages {
    public static final int HEADER_SIZE = 5;
    private byte[] buffer = new byte[0];
    private int startIndex = 0;

    public static byte[] concatinateBytes(byte[] left, byte[] right) {
        byte[] newArray = new byte[left.length + right.length];
        System.arraycopy(left, 0, newArray, 0, left.length);
        System.arraycopy(right, 0, newArray, left.length, right.length);

        return newArray;
    }

    public static <K, V extends Comparable<? super V>> Map<K, V> sortByValue(Map<K, V> map) {
        List<Map.Entry<K, V>> list =
                new LinkedList<>(map.entrySet());
        Collections.sort(list, new Comparator<Map.Entry<K, V>>() {
            @Override
            public int compare(Map.Entry<K, V> o1, Map.Entry<K, V> o2) {
                return (o1.getValue()).compareTo(o2.getValue());
            }
        });

        Map<K, V> result = new LinkedHashMap<>();
        for (Map.Entry<K, V> entry : list) {
            result.put(entry.getKey(), entry.getValue());
        }
        return result;
    }

    public void push(byte[] message) {
        this.buffer = concatinateBytes(this.buffer, message);
    }

    private void bytesToString(byte[] firstBytes, String prefix) {
        StringBuilder sb = new StringBuilder();
        for (byte b : firstBytes) {
            sb.append(String.format("%02X ", b));
        }
    }

    public ArrayList<byte[]> splitMessages() {
        ArrayList<byte[]> messages = new ArrayList<>();
        getMessage(this.buffer, messages);
        return messages;
    }

    private void getMessage(byte[] buffer, ArrayList<byte[]> messages) {
        byte[] bufferCopy = Arrays.copyOf(buffer, buffer.length);
        int headerEnd = startIndex + HEADER_SIZE;

        if (headerEndIndexIsInsideTheBuffer(buffer, headerEnd)) {
            int msgLength = getMsgLength(bufferCopy, headerEnd);
            if (isEntireMessageInTheBuffer(buffer, msgLength)) {
                messages.add(extractMessageFromBuffer(bufferCopy, msgLength));
                moveStartIndexToNextMessage(msgLength);
                getMessage(buffer, messages);
            }
        }
    }

    private byte[] extractMessageFromBuffer(byte[] buffer, int msgLength) {
        return Arrays.copyOfRange(buffer, startIndex, startIndex + msgLength);
    }

    private boolean isEntireMessageInTheBuffer(byte[] buffer, int msgLength) {
        return (buffer.length - startIndex) >= msgLength;
    }

    private void moveStartIndexToNextMessage(int msgLength) {
        startIndex = startIndex + msgLength;
    }

    private int getMsgLength(byte[] bufferCopy, int headerEnd) {
        byte[] messageHeader = Arrays.copyOfRange(bufferCopy, startIndex, headerEnd);
        return GazelleMessageParser.getMsgLength(messageHeader);
    }

    private boolean headerEndIndexIsInsideTheBuffer(byte[] buffer, int headerEnd) {
        return headerEnd < buffer.length;
    }
}
