package net.ihe.gazelle.proxy.netty.protocols.tlsRaw;

import net.ihe.gazelle.security.ssl.GazelleMessageParser;
import org.jboss.netty.buffer.ChannelBuffer;
import org.jboss.netty.channel.*;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.slf4j.MDC;

import java.util.ArrayList;
import java.util.Arrays;

/**
 * Created by gthomazon on 31/08/16.
 */
public class TlsRawDecoder extends SimpleChannelUpstreamHandler {

    private static Logger LOG = LoggerFactory.getLogger(TlsRawDecoder.class);
    private GazelleMessageParser tlsMessageParser;
    private HandleReceivedMessages handleReceivedMessages = new HandleReceivedMessages();

    public TlsRawDecoder() {
    }

    public TlsRawDecoder(GazelleMessageParser message) {
        this.tlsMessageParser = message;
    }

    @Override
    public void messageReceived(ChannelHandlerContext ctx, MessageEvent e) throws Exception {

        MDC.put("username", ctx.getChannel().getRemoteAddress().toString());


        Object m = e.getMessage();
        if (!(m instanceof ChannelBuffer)) {
            ctx.sendUpstream(e);
            return;
        }

        ChannelBuffer input = (ChannelBuffer) m;
        if (!input.readable()) {
            return;
        }

        try {
            byte[] buf = new byte[input.readableBytes()];
            input.readBytes(buf);

            handleReceivedMessages.push(buf);

            ArrayList<byte[]> messages = handleReceivedMessages.splitMessages();
            for (byte[] message : messages) {
                Channels.fireMessageReceived(ctx, message);
            }

        } catch (Exception ex) {
            LOG.error("messageReceived : " + ex.getMessage());
        }

    }

    public void splitMessages(ArrayList<byte[]> messages, byte[] completeBuff) {
        byte[] buf;
        buf = Arrays.copyOf(completeBuff, completeBuff.length);

        int messSize = getMessageSize(buf);

        messages.add(Arrays.copyOfRange(buf, 0, messSize));

        int bufSize = buf.length;
        while (messSize < bufSize) {
            byte[] firstBytes;
            firstBytes = Arrays.copyOfRange(buf, messSize, messSize + 5);
            int size2 = tlsMessageParser.getMsgLength(firstBytes);

            messages.add(Arrays.copyOfRange(buf, messSize, messSize + size2));

            messSize = messSize + size2;
        }
    }

    private int getMessageSize(byte[] buf) {
        byte[] firstBytes = Arrays.copyOfRange(buf, 0, 5);
        return tlsMessageParser.getMsgLength(firstBytes);
    }

    @Override
    public void channelUnbound(ChannelHandlerContext ctx, ChannelStateEvent e) throws Exception {
        super.channelUnbound(ctx, e);
    }

    @Override
    public void channelDisconnected(ChannelHandlerContext ctx, ChannelStateEvent e) throws Exception {
        super.channelDisconnected(ctx, e);
    }

    @Override
    public void channelClosed(ChannelHandlerContext ctx, ChannelStateEvent e) throws Exception {
        super.channelClosed(ctx, e);
    }

    @Override
    public void exceptionCaught(ChannelHandlerContext ctx, ExceptionEvent e) throws Exception {
        ctx.sendUpstream(e);
    }

}
