package net.ihe.gazelle.proxy.netty.protocols.tlsRaw;

import net.ihe.gazelle.proxy.netty.ConnectionConfig;
import net.ihe.gazelle.proxy.netty.Proxy;
import net.ihe.gazelle.proxy.netty.ProxyEventListener;
import net.ihe.gazelle.security.ssl.GazelleMessageParser;
import net.ihe.gazelle.security.ssl.GazelleTlsMsgToXml;
import org.jboss.netty.channel.ChannelHandler;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.ArrayList;
import java.util.List;

public class TlsRawProxy extends Proxy<byte[], byte[]> {
    private static Logger LOG = LoggerFactory.getLogger(TlsRawProxy.class);

    private GazelleTlsMsgToXml exporter = new GazelleTlsMsgToXml();
    private GazelleMessageParser tlsMessageParser = new GazelleMessageParser(exporter);

    public TlsRawProxy(ProxyEventListener<byte[], byte[]> proxyEventListener, ConnectionConfig connectionConfig) {
        super(proxyEventListener, connectionConfig);
    }

    @Override
    public List<ChannelHandler> getDecodersForRequest() {
        List<ChannelHandler> channels = new ArrayList<ChannelHandler>();

        // Decode TLS RAW messages
        channels.add(new TlsRawDecoder(tlsMessageParser));

        return channels;
    }

    @Override
    public List<ChannelHandler> getDecodersForResponse() {
        List<ChannelHandler> channels = new ArrayList<ChannelHandler>();

        // Decode TLS RAW messages
        channels.add(new TlsRawDecoder(tlsMessageParser));

        return channels;
    }

    @Override
    public byte[] handleRequest(Object message) {
        if (message instanceof byte[]) {
            return (byte[]) message;
        }
        return null;
    }

    @Override
    public byte[] handleResponse(Object message) {
        return handleRequest(message);
    }

}
